"""
Generate latex output comparing time series of given sww files

Inputs:

output dirs: dictionary of output directories with a
             association to simulation, eg 50m boundary, 100m boundary
                    
Outputs:

* latex output to generate figures for report 
* figures used for report stored in the report_figure directory
NOTE, this directory will need to be committed, as well as
the latex files.

June 2006                    
"""

from os import sep
import project
from anuga.pyvolution.util import sww2timeseries, get_gauges_from_file

# User defined inputs

# for comparison between DTED and WA DLI data
production_dirs = {'20060515_001733': 'DTED',
                   '20060704_063234': 'WA DLI'}

plot_quantity = ['stage', 'speed']

# Create sections and graphs for each designated production directory
latex_output = []
swwfiles = {}
for label_id in production_dirs.keys():
    
    file_loc = project.outputdir + label_id + sep
    swwfile = file_loc + project.basename + '.sww'
    swwfiles[swwfile] = label_id

texname, vec = sww2timeseries(swwfiles,
                              project.gauge_filename,
                              production_dirs,
                              report = True,
                              reportname = 'compare_output_datasets',
                              surface = False,
                              plot_quantity = plot_quantity,
                              time_min = None,
                              time_max = None,
                              title_on = False,
                              verbose = True)

latex_output.append(texname)
